/*
 * BK Id: SCCS/s.open_pic.h 1.22 10/11/01 13:10:14 trini
 */
/*
 *  arch/ppc/kernel/open_pic.h -- OpenPIC Interrupt Handling
 *
 *  Copyright (C) 1997 Geert Uytterhoeven
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License.  See the file COPYING in the main directory of this archive
 *  for more details.
 *  
 */

#ifndef _PPC_KERNEL_OPEN_PIC_H
#define _PPC_KERNEL_OPEN_PIC_H

#include <linux/config.h>

#define OPENPIC_SIZE	0x40000

/*
 *  Non-offset'ed vector numbers
 */

#define OPENPIC_VEC_TIMER	64	/* and up */
#define OPENPIC_VEC_IPI		72	/* and up */
#define OPENPIC_VEC_SPURIOUS	127

/* OpenPIC IRQ controller structure */
extern struct hw_interrupt_type open_pic;

/* OpenPIC IPI controller structure */
#ifdef CONFIG_SMP
extern struct hw_interrupt_type open_pic_ipi;
#endif /* CONFIG_SMP */

extern void* OpenPIC_Addr;
#if 0
extern u_int OpenPIC_NumInitSenses;
extern u_char *OpenPIC_InitSenses;
/* The following mainly for the MPC107, which has a "builtin" offset for the Source registers
	Setting this to 16 will make all the routines work for a MPC107
*/
extern u_int OpenPIC_RegOffset;
#endif


extern inline int openpic_to_irq(int irq)
{
	/* IRQ 0 usually means 'disabled'.. don't mess with it 
	 * exceptions to this (sandpoint maybe?) 
	 * shouldn't use openpic_to_irq 
	 */
	if (irq != 0){
		return irq += NUM_8259_INTERRUPTS;
	} else {
		return 0;
	}
}
/*extern int open_pic_irq_offset;*/

typedef enum irq_polarity_en {
	OP_IRQ_POS = 1, 
	OP_IRQ_NEG = 0
} irq_polarity_type;

typedef enum irq_sense_en {
	OP_IRQ_LEVEL = 1,
	OP_IRQ_EDGE  = 0
} irq_sense_type;

typedef struct openpic_irq_def_str {
	int					PICIrq;		/* Must be int (not u_int) to mark end of table */
	u_int				Vector;
	u_char				Priority;
	irq_sense_type		IrqSense;
	irq_polarity_type	IrqPolarity;
	/* This can be used for cascade, but should work well for the chrp specials too */
	int					(*CascadeAckHandler)(int);
} openpic_irq_def;

typedef struct openpic_def_str {
	volatile struct OpenPIC	*OpenPIC_Addr;
	int				slave_pic;
	openpic_irq_def	*IRQdef;	/* The lovest vector first in the table !!! */
} openpic_def;

#define ALLOC_OPENPIC_DEF(name) static openpic_def name __initdata = {NULL, 0, NULL}

/* Exported functions */
extern void openpic_oldinit(int, int, unsigned char *, int);
extern void openpic_init(openpic_def *);
extern u_int openpic_irq(void);
extern void openpic_eoi(void);
extern void openpic_request_IPIs(void);
extern void do_openpic_setup_cpu(void);
extern int openpic_get_irq(struct pt_regs *regs);
extern void openpic_reset_processor_phys(u_int cpumask);
extern void openpic_setup_ISU(int isu_num, unsigned long addr);
extern void openpic_cause_IPI(u_int ipi, u_int cpumask);
extern void smp_openpic_message_pass(int target, int msg, unsigned long data,
				     int wait);

#endif /* _PPC_KERNEL_OPEN_PIC_H */
