/*
 * arch/ppc/kernel/vg4_pci.c
 * 
 * PCI support (including autoconfiguration) for SBS VG4
 *
 * Author: Matt Porter
 *         mporter@mvista.com
 *
 * Modified for VG4 by Dag Nygren 2001 (dag@newtech.fi)
 *
 * Copyright 2000 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR   IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/malloc.h>

#include <asm/byteorder.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/machdep.h>
/* #include <asm/vg4.h> */
#include <asm/pci-bridge.h>

#include "pci.h"
#include "pci_auto.h"
#include "i8259.h"
#include "open_pic2.h"
#include "mpc10x.h"


static inline int __init
vg4_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
        int irq;
		/* SBS VG4 interrupt routes */
		char pci_irq_table[][4] =
		{
			{18,   19,     20,     21},     /* IDSEL 12 - 91C142 VME controller */
			{17,   18,     19,     20},     /* IDSEL 13 - 21143 Ethernet */
			{19,    0,      0,      0},     /* IDSEL 14 - 53C825 SCSI */
			{0,     0,      0,      0},     /* IDSEL 15 - PMC1 */
			{0,     0,      0,      0},     /* IDSEL 16 - PMC2 */
			{0,     0,      0,      0},		/* IDSEL 17 - MPC107 Northbridge*/
			{0,     0,      0,      0},		/* IDSEL 18 - PCI-ISA, M1543C Southbridge*/
			{0,     0,      0,      0},     /* IDSEL 19 - unused */
			{0,     0,      0,      0},     /* IDSEL 20 - unused */
			{0,     0,      0,      0},     /* IDSEL 21 - unused */
			{0,     0,      0,      0},     /* IDSEL 22 - unused */
			{0,     0,      0,      0},     /* IDSEL 23 - unused */
			{0,     0,      0,      0},     /* IDSEL 24 - unused */
			{0,     0,      0,      0},     /* IDSEL 25 - unused */
			{0,     0,      0,      0},     /* IDSEL 26 - unused */
			{0,     0,      0,      0},     /* IDSEL 27 - unused */
			{0,     0,      0,      0},		/* IDSEL 28 - Power Management, M1543C Southbridge*/
		};
		const long min_idsel = 12, max_idsel = 28, irqs_per_slot = 4;

// printk("Mapping idesel %d, pin %d => ", idsel, pin);
        irq = PCI_IRQ_TABLE_LOOKUP;
        if (!irq)
                return 0;

// printk("%d\n", irq);
        return irq;
}


void __init
vg4_find_bridges(void)
{
    struct pci_controller* hose;
 
    hose = pcibios_alloc_controller();
    if (!hose)
        return;
 
    hose->first_busno = 0;
    hose->last_busno = 0xff;
	if (mpc10x_bridge_init(hose,
			MPC10X_MEM_MAP_A,
			MPC10X_MEM_MAP_A,
			MPC10X_MAPA_EUMB_BASE) == 0) {

		hose->mem_resources[0].end = 0xffffffff;
		hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);
		ppc_md.pci_swizzle = common_swizzle;
		ppc_md.pci_map_irq = vg4_map_irq;
	}
}

