/*
 * arch/ppc/kernel/vg4_setup.c
 * 
 * Board setup routines for SBS VG4
 *
 * Based on the K2 setup by:
 * Author: Matt Porter
 *         mporter@mvista.com
 *
 * Heavily modified for VG4: 
 *					Dag Nygren
 *					Oy Espoon Newtech Ab
 *					dag@newtech.fi
 *
 * Copyright 2000 MontaVista Software Inc.
 * VG4 modifications: Copyright 2001 Oy Espoon Newtech Ab
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR   IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h> 
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/processor.h>

#include <asm/time.h>
#include "vg4.h"
#include "i8259.h"
#include "open_pic2.h"
#include "mpc10x.h"
#include "todc.h"

extern void vg4_find_bridges(void);

static void
inline vg4_cardfail_led(int state) {
	outb(VG4_LOCKR_UNLOCK, VG4_LOCKR); 
	outb(state, VG4_FAIL_LED); 
	outb(VG4_LOCKR_LOCK, VG4_LOCKR); 
}

void
vg4_user_led(int state) { outb(state, VG4_USER_LED); }

static int
vg4_get_bus_speed(void)
{
	int bus_speed;

/* Is there a nice way to figure out this ? */
	bus_speed = 100000000;
	return bus_speed;
}

static int
vg4_get_cpu_speed(void)
{
	unsigned long hid1;
	int cpu_speed;
	unsigned int cpu_7xx[16] = {
		0, 15, 14, 0, 0, 13, 5, 9, 6, 11, 8, 10, 16, 12, 7, 0
	};
	unsigned int cpu_6xx[16] = {
		0, 0, 14, 0, 0, 13, 5, 9, 6, 11, 8, 10, 0, 12, 7, 0
	};

	hid1 = mfspr(HID1) >> 28;

	if ( (mfspr(PVR)>>16) == 8)
		hid1 = cpu_7xx[hid1];
	else
		hid1 = cpu_6xx[hid1];

	cpu_speed = vg4_get_bus_speed()*hid1/2;
	return cpu_speed;
}

static char *
vg4_get_l2_size(void)
{
/* Is there a nice way to figure out this ? */
	return ("1MB");
}

static int
vg4_get_cpuinfo(char *buffer)
{
	int len;

	len =  sprintf( buffer,     "vendor\t\t: SBS\n");
	len += sprintf( buffer+len, "machine\t\t: VG4\n");
	len += sprintf( buffer+len, "cpu speed\t: %dMhz\n",
		vg4_get_cpu_speed()/1000000);
	len += sprintf( buffer+len, "bus speed\t: %dMhz\n",
		vg4_get_bus_speed()/1000000);
	len += sprintf( buffer+len, "L2\t\t: %s\n", vg4_get_l2_size());
	len += sprintf( buffer+len, "memory type\t: SDRAM\n"); 
	len += sprintf( buffer+len, "\n");

	return len;
}

static void __init
		vg4_setup_arch(void)
{
	unsigned int cpu;
	extern char cmd_line[];

	/* init to some ~sane value until calibrate_delay() runs */
    loops_per_jiffy = 50000000/HZ;

	/* Lookup PCI host bridges */
	vg4_find_bridges();

#ifdef CONFIG_ROOT_NFS
	/* bootable from nfsroot */
	ROOT_DEV = to_kdev_t(0x00FF);	/* /dev/nfs pseudo device */
#else
	/* bootable from SCSI second partition */
	ROOT_DEV = to_kdev_t(0x0802);	/* /dev/sda2 */
#endif

	printk("Boot arguments: %s\n", cmd_line);

	/* Identify the system */
	printk("System Identification: SBS VG4 - PowerPC 750 @ %d Mhz\n", vg4_get_cpu_speed()/1000000);

	/* Identify the CPU manufacturer */
	cpu =  mfspr(PVR);
	printk("CPU manufacturer: %s [rev=%04x]\n", (cpu & (1<<15)) ? "IBM" :
	       "Motorola", (cpu & 0xffff));
}

static void
vg4_restart(char *cmd)
{
	__cli();
	vg4_cardfail_led(VG4_FLED_ON);
	/* SRR0 has system reset vector, SRR1 has default MSR value */
	/* rfi restores MSR from SRR1 and sets the PC to the SRR0 value */
	__asm__ __volatile__
	("\n\
	lis	3,0xfff0
	ori	3,3,0x0100
	mtspr	26,3
	li	3,0
	mtspr	27,3
	rfi
	");
	for(;;);
}

static void
vg4_power_off(void)
{
	for(;;);
}

static void
vg4_halt(void)
{
	__cli();
	while (1);
}

	/* Using MPC107(EPIC) means that the LinuxVector numbers start from 16
	 * as 0-15 is reserved for the possible cascade
	 * Remember that the LinuxVector of the first entry is used as an offset
	 * for the Timer and IPI vectors */

static openpic_irq_def vg4_openpic_irq_defs[] __initdata = {
	/* PICirq, LinuxVector, Priority, Level/Edge, Polarity, CascadeAck */
	{  16, 16, 8, OP_IRQ_LEVEL, OP_IRQ_POS, i8259_irq} , 	/* 8259 cascade */
	{  17, 17, 8, OP_IRQ_LEVEL, OP_IRQ_NEG, NULL      } , 	/* INT_PCI_INTA */
	{  18, 18, 8, OP_IRQ_LEVEL, OP_IRQ_NEG, NULL      } , 	/* INT_PCI_INTB */
	{  19, 19, 8, OP_IRQ_LEVEL, OP_IRQ_NEG, NULL      } , 	/* INT_PCI_INTC */
	{  20, 20, 8, OP_IRQ_LEVEL, OP_IRQ_NEG, NULL      } ,	/* INT_PCI_INTD */
	{ -1, -1, 8, OP_IRQ_LEVEL, OP_IRQ_NEG, NULL      }  	/* End of table marker */
};   

ALLOC_OPENPIC_DEF(vg4_openpic_def);

static void __init 
vg4_init_IRQ(void)
{
    int vec;

	vg4_openpic_def.IRQdef    = vg4_openpic_irq_defs;
 
    /* The int:s behind the OpenPIC in MPC107 */

    openpic_init(&vg4_openpic_def);
 
    /* These are the ints (0-15) behind the 8259 in the ALI chip */
    for ( vec = 0 ; vec < NUM_8259_INTERRUPTS  ; vec++ )
        irq_desc[vec].handler = &i8259_pic;
 
    i8259_init();
}


static void __init
vg4_init2(void)
{
	request_region(0x20,0x20,"pic1");
	request_region(0xa0,0x20,"pic2");
	request_region(0x00,0x20,"dma1");
	request_region(0x40,0x20,"timer");
	request_region(0x80,0x10,"dma page reg");
	request_region(0xc0,0x20,"dma2");
	request_region(0x160,0x10,"VG4 ext reg set");
	request_region(VG4_USER_LED,0x01,"User LED");
}

/*
 * Set BAT 3 to map 0x80000000. (For access to the mpc107)
 */
static __inline__ void
vg4_set_bat(void)
{
        unsigned long   bat3u, bat3l;
        static int      mapping_set = 0;

        if (!mapping_set)
        {

                /* wait for all outstanding memory accesses to complete */
                mb();

                /* setup DBATs */
                mtspr(DBAT3U, 0x80001ffe);
                mtspr(DBAT3L, 0x8000002a);

                /* wait for updates */
                mb();

                mapping_set = 1;
        }
        return;
}

static unsigned long __init
vg4_find_end_of_memory(void)
{
	/* Cover I/O space with a BAT */
	/* yuck, better hope your ram size is a power of 2  -- paulus */
	vg4_set_bat();
	return mpc10x_get_mem_size(MPC10X_MEM_MAP_A);
}

static void __init
vg4_map_io(void)
{
	io_block_mapping(VG4_PCI_IO_BASE, VG4_PCI_PHY_IO_BASE, 0x10000000, _PAGE_IO);
	io_block_mapping(VG4_PCI_MEM_BASE, VG4_PCI_PHY_MEM_BASE, 0x10000000, _PAGE_IO);
	ioremap_base = VG4_PCI_MEM_BASE;
}

TODC_ALLOC();

void __init
			platform_init(unsigned long r3, unsigned long r4, unsigned long r5,
			unsigned long r6, unsigned long r7)
{
	isa_io_base = MPC10X_MAPA_ISA_IO_BASE;
	isa_mem_base = MPC10X_MAPA_ISA_MEM_BASE;
 	pci_dram_offset = MPC10X_MAPA_DRAM_OFFSET;

#if 0
	ISA_DMA_THRESHOLD = 0x00ffffff;
	DMA_MODE_READ = 0x44;
	DMA_MODE_WRITE = 0x48;
#endif

#ifdef CONFIG_BLK_DEV_INITRD
	if ( r4 )
	{
		initrd_start = r4 + KERNELBASE;
		initrd_end = r5 + KERNELBASE;
	}
#endif

	/* take care of cmd line */
	if ( r6  && (((char *) r6) != '\0'))
	{
		*(char *)(r7+KERNELBASE) = 0;
		strcpy(cmd_line, (char *)(r6+KERNELBASE));
	} 

#if 0
	/* Set up a default rootdevice (Temporary mod) */
	if (!strstr(cmd_line, "root="))
		strcat(cmd_line, " root=/dev/sda3");
#endif

	ppc_md.setup_arch = vg4_setup_arch;
	ppc_md.get_cpuinfo = vg4_get_cpuinfo;

	ppc_md.init_IRQ = vg4_init_IRQ;
	ppc_md.get_irq = openpic_get_irq;
	ppc_md.init = vg4_init2;

	ppc_md.find_end_of_memory = vg4_find_end_of_memory;
	ppc_md.setup_io_mappings = vg4_map_io;

	ppc_md.restart = vg4_restart;
	ppc_md.power_off = vg4_power_off;
	ppc_md.halt = vg4_halt;

	/* VG4 has an MC146818 RTC */
	TODC_INIT(TODC_TYPE_MC146818,
		VG4_NVRAM_RTC_ADDR,0,
		VG4_NVRAM_RTC_DATA,8);

	ppc_md.time_init = todc_time_init;
	ppc_md.set_rtc_time = todc_set_rtc_time;
	ppc_md.get_rtc_time = todc_get_rtc_time;
	ppc_md.nvram_read_val = todc_mc146818_read_val;
	ppc_md.nvram_write_val = todc_mc146818_write_val;
	ppc_md.calibrate_decr = todc_calibrate_decr;
}
